@@ -1,8 +1,8 @@
 # $File: //member/autrijus/PAR/lib/PAR.pm $ $Author: autrijus $
-# $Revision: #14 $ $Change: 6210 $ $DateTime: 2003/05/31 12:42:06 $ vim: expandtab shiftwidth=4
+# $Revision: #26 $ $Change: 7234 $ $DateTime: 2003/07/29 06:38:20 $ vim: expandtab shiftwidth=4
 
 package PAR;
-$PAR::VERSION = '0.69';
+$PAR::VERSION = '0.70';
 
 use 5.006;
 use strict;
@@ -15,12 +15,12 @@ PAR - Perl Archive Toolkit
 
 =head1 VERSION
 
-This document describes version 0.69 of PAR, released May 31, 2003.
+This document describes version 0.70 of PAR, released July 29, 2003.
 
 =head1 SYNOPSIS
 
 (If you want to make an executable that contains all module, scripts and
-data files, please consult L<pp> instead.)
+data files, please consult the bundled L<pp> utility instead.)
 
 Following examples assume a F<foo.par> file in Zip format; support for
 compressed tar (F<*.tgz>/F<*.tbz2>) format is under consideration.
@@ -44,6 +44,15 @@ Following paths inside the PAR file are searched:
     /5.8.0/i386-freebsd/        # both of the above
     /
 
+PAR files may also (recursively) contain other PAR files.
+All files under following paths will be considered as PAR
+files and searched as well:
+
+    /par/i386-freebsd/          # i.e. $Config{archname}
+    /par/5.8.0/                 # i.e. $Config{version}
+    /par/5.8.0/i386-freebsd/    # both of the above
+    /par/
+
 Run F<script/test.pl> or F<test.pl> from F<foo.par>:
 
     % perl -MPAR foo.par test.pl        # only when $0 ends in '.par'
@@ -89,12 +98,12 @@ under F<arch/> and F<lib/>, e.g.:
 Afterward, you can just use F<mymodule.par> anywhere in your C<@INC>,
 use B<PAR>, and it will Just Work.
 
-For maximal convenience, you can set the C<PERL5OPT> environment
-variable to C<-MPAR> to enable C<PAR> processing globally (the overhead
-is small if not used), or to C<-MPAR=/path/to/mylib.par> to load a
+For convenience, you can set the C<PERL5OPT> environment variable to
+C<-MPAR> to enable C<PAR> processing globally (the overhead is small
+if not used); setting it to C<-MPAR=/path/to/mylib.par> will load a
 specific PAR file.  Alternatively, consider using the F<par.pl> utility
-bundled with this module, or the self-contained F<parl> utility on
-machines without PAR.pm installed.
+bundled with this module, or using the self-contained F<parl> utility
+on machines without PAR.pm installed.
 
 Note that self-containing scripts and executables created with F<par.pl>
 and F<pp> may also be used as F<.par> archives:
@@ -141,12 +150,12 @@ sub import {
         if ($par =~ /[?*{}\[\]]/) {
             require File::Glob;
             foreach my $matched (File::Glob::glob($par)) {
-                push @PAR_INC, $matched if unpar($matched, undef, undef, 1);
+                push @PAR_INC, unpar($matched, undef, undef, 1);
             }
             next;
         }
 
-        push @PAR_INC, $par if unpar($par, undef, undef, 1);
+        push @PAR_INC, unpar($par, undef, undef, 1);
     }
 
     unshift @INC, \&find_par unless grep { $_ eq \&find_par } @INC;
@@ -183,11 +192,16 @@ sub import {
 
 sub _run_member {
     my $member = shift;
+    my $clear_stack = shift;
     my ($fh, $is_new) = _tmpfile($member->crc32String . ".pl");
 
     if ($is_new) {
         my $file = $member->fileName;
-        print $fh "package main; shift \@INC;\n#line 1 \"$file\"\n";
+        print $fh "package main; shift \@INC;\n";
+        if (defined &Internals::PAR_CLEARSTACK and $clear_stack) {
+            print $fh "Internals::PAR_CLEARSTACK();\n";
+        }
+        print $fh "#line 1 \"$file\"\n";
         $member->extractToFileHandle($fh);
         seek ($fh, 0, 0);
     }
@@ -201,7 +215,15 @@ sub _run_member {
 sub find_par {
     my ($self, $file, $member_only) = @_;
 
-    foreach my $path (@PAR_INC ? @PAR_INC : @INC) {
+    my $scheme;
+    foreach (@PAR_INC ? @PAR_INC : @INC) {
+        my $path = $_;
+        if (!@PAR_INC and $path and $path =~ m!//! and $scheme and $scheme =~ /^\w+$/) {
+            $path = "$scheme:$path";
+        }
+        else {
+            $scheme = $path;
+        }
         my $rv = unpar($path, $file, $member_only, 1);
         return $rv if defined($rv);
     }
@@ -225,15 +247,38 @@ sub par_handle {
     return $LibCache{$par};
 }
 
+my %escapes;
 sub unpar {
     my ($par, $file, $member_only, $allow_other_ext) = @_;
     my $zip = $LibCache{$par};
+    my @rv = $par;
 
     return if $PAR::__reading;
     local $PAR::__reading = 1;
 
     unless ($zip) {
-        unless (($allow_other_ext or $par =~ /\.par\z/i) and -f $par) {
+        if ($par =~ m!^\w+://!) {
+            require File::Spec;
+            require LWP::Simple;
+            $ENV{PAR_TEMP} ||= File::Spec->catdir(File::Spec->tmpdir, 'par');
+            mkdir $ENV{PAR_TEMP}, 0777;
+
+            my $file = $par;
+            if (!%escapes) {
+                $escapes{chr($_)} = sprintf("%%%02X", $_) for 0..255;
+            }
+            $file =~ s/([^\w\.])/$escapes{$1}/g;
+            $file = File::Spec->catfile( $ENV{PAR_TEMP}, $file);
+            LWP::Simple::mirror( $par, $file );
+            return unless -e $file;
+            $par = $file;
+        }
+        elsif (ref($par) eq 'SCALAR') {
+            my ($fh) = _tmpfile();
+            print $fh $$par;
+            $par = $fh;
+        }
+        elsif (!(($allow_other_ext or $par =~ /\.par\z/i) and -f $par)) {
             $par .= ".par";
             return unless -f $par;
         }
@@ -242,13 +287,26 @@ sub unpar {
         require Archive::Zip;
 
         $zip = Archive::Zip->new;
-        return unless $zip->read($par) == Archive::Zip::AZ_OK();
+        my $method = (ref($par) ? 'readFromFileHandle' : 'read');
+        return unless $zip->$method($par) == Archive::Zip::AZ_OK();
 
         push @LibCache, $zip;
-        $LibCache{$par} = $zip;
+        $LibCache{$_[0]} = $zip;
+
+        foreach my $member ( $zip->members(
+            "^par/(?:$Config::Config{version}/)?(?:$Config::Config{archname}/)?"
+        ) ) {
+            next if $member->isDirectory;
+            my $content = $member->contents();
+            next unless $content =~ /^PK\003\004/;
+            local $PAR::__reading = 0;
+            push @rv, unpar(\$content, undef, undef, 1);
+        }
     }
 
-    return 1 unless defined $file;
+    $PAR::LastAccessedPAR = $zip;
+
+    return @rv unless defined $file;
 
     my $member = $zip->memberNamed("lib/$file")
               || $zip->memberNamed("arch/$file")
@@ -280,7 +338,8 @@ sub _tmpfile {
     # at the time File::Temp calls unlink, there is still an open handle around,
     # and Win32 can't delete opened files...
     #
-    if ($ENV{PAR_CLEARTEMP}) {
+    if ($ENV{PAR_CLEARTEMP} or !@_) {
+        require IO::File;
         my $fh = IO::File->new_tmpfile;
         unless( $fh ) {
             require File::Temp;
@@ -312,6 +371,8 @@ sub _tmpfile {
 
 =head1 SEE ALSO
 
+L<http://www.autrijus.org/par-tutorial/>
+
 L<PAR::Intro>
 
 L<par.pl>, L<parl>, L<pp>
