@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 # $File: //member/autrijus/PAR/myldr/file2c.pl $ $Author: autrijus $
-# $Revision: #6 $ $Change: 5898 $ $DateTime: 2003/05/16 16:30:10 $
+# $Revision: #13 $ $Change: 7237 $ $DateTime: 2003/07/29 07:19:11 $
 #
 # Copyright (c) 2002 Mattia Barbon.
 # Copyright (c) 2002 Autrijus Tang.
@@ -13,6 +13,7 @@ my $give_help = 0;
 my $pl_file = shift;
 my $c_file = shift;
 my $c_var = shift;
+my $long_literal = shift;
 
 $give_help ||= ( !defined $pl_file or
                 !defined $c_file or
@@ -37,20 +38,28 @@ undef $/;
 my $pl_text = <IN>;
 close IN;
 
-$pl_text = pod_strip($pl_text, basename($pl_file)) if -e $pl_file;
+$pl_text = pod_strip($pl_text, basename($pl_file)) if -e $pl_file and $pl_file =~ /\.p[lm]/i;
+$pl_text = reverse $pl_text;
 
 #  make a c-array
-sub map_fun { local $_ = $_[0];
-              m/[\\"']/ and return "\\$_";
-              ord() >= 32 && ord() <= 127 && return $_;
-              return sprintf '\0%o', ord };
 
-my @c_chars = map { map_fun($_) } split '', $pl_text;
-my $c_arr = "static char $c_var\[] = { " .
-  ( join ', ', map { "'$_'" } @c_chars ) .
-  ", '\\0' };\n";
+print OUT "const char * name_$c_var = \"" . basename($pl_file) . "\";\n";
+print OUT "unsigned long size_$c_var = " . length($pl_text) . ";\n";
+print OUT "const char $c_var\[" . (length($pl_text) + 1) . "] = ";
+print OUT $long_literal ? '"' : '{';
 
-print OUT $c_arr;
+my $i;
+for (1..length($pl_text)) {
+    if ($long_literal) {
+	print OUT sprintf '\%03o', ord(chop($pl_text));
+    }
+    else {
+	print OUT sprintf "'\\%03o',", ord(chop($pl_text));
+	print OUT "\n" unless $i++ % 16;
+    }
+}
+
+print OUT $long_literal ? "\";\n" : "0\n};\n";
 close OUT;
 
 sub pod_strip {
@@ -72,6 +81,7 @@ sub pod_strip {
     }gsex;
     $pl_text = '#line 1 "' . ($filename) . "\"\n" . $pl_text
         if length $filename;
+    $pl_text =~ s/^#line 1 (.*\n)(#!.*\n)/$2#line 2 $1/g;
 
     return $pl_text;
 }
