@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # $File: //member/autrijus/PAR/script/par.pl $ $Author: autrijus $
-# $Revision: #63 $ $Change: 6169 $ $DateTime: 2003/05/29 18:53:54 $ vim: expandtab shiftwidth=4
+# $Revision: #69 $ $Change: 7232 $ $DateTime: 2003/07/29 03:55:38 $ vim: expandtab shiftwidth=4
 
 package __par_pl;
 
@@ -16,6 +16,19 @@ par.pl - Make and run Perl Archives
 (Please see L<pp> for convenient ways to make self-contained
 executables, scripts or PAR archives from perl programs.)
 
+To make a I<PAR distribution> from a CPAN module distribution:
+
+    % par.pl -p                 # make a PAR dist under the current path
+    % par.pl -p Foo-0.01        # assume unpacked CPAN dist in Foo-0.01/
+
+To manipulate a I<PAR distribution>:
+
+    % par.pl -i Foo-0.01-i386-freebsd-5.8.0.par     # install
+    % par.pl -i http://example.com/foo.par          # also works
+    % par.pl -u Foo-0.01-i386-freebsd-5.8.0.par     # uninstall
+    % par.pl -s Foo-0.01-i386-freebsd-5.8.0.par     # sign
+    % par.pl -v Foo-0.01-i386-freebsd-5.8.0.par     # verify
+
 To use F<Hello.pm> from F<./foo.par>:
 
     % par.pl -A./foo.par -MHello 
@@ -57,6 +70,10 @@ This stand-alone command offers roughly the same feature as C<perl
 -MPAR>, except that it takes the pre-loaded F<.par> files via
 C<-Afoo.par> instead of C<-MPAR=foo.par>.
 
+Additionally, it lets you convert a CPAN distribution to a PAR
+distribution, as well as manipulate such distributions.  For more
+information about PAR distributions, see L<PAR::Dist>.
+
 =head2 Binary PAR loader (L<parl>)
 
 If you have a C compiler, or a pre-built binary package of B<PAR> is
@@ -205,7 +222,7 @@ my ($start_pos, $data_pos);
         elsif ( $fullname =~ m|^/?shlib/| and defined $ENV{PAR_TEMP} ) {
             # should be moved to _tempfile()
             $filename = "$ENV{PAR_TEMP}/$basename$ext";
-            print "SHLIB: $filename\n";
+            outs("SHLIB: $filename\n");
             open $out, '>', $filename or die $!;
             binmode($out);
             print $out $buf;
@@ -230,7 +247,7 @@ my ($start_pos, $data_pos);
                 next unless /\Q$module\E$/;
                 $key = $_; last;
             }
-            delete $require_list{$key};
+            delete $require_list{$key} if defined($key);
         } or return;
 
         $INC{$module} = "/loader/$filename/$module";
@@ -291,8 +308,15 @@ $quiet = 0 unless $ENV{PAR_DEBUG};
 
 # Don't swallow arguments for compiled executables without --par-options
 if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
+    my %dist_cmd = qw(
+        p   blib_to_par
+        i   install_par
+        u   uninstall_par
+        s   sign_par
+        v   verify_par
+    );
     while (@ARGV) {
-        $ARGV[0] =~ /^-([AIMOBbq])(.*)/ or last;
+        $ARGV[0] =~ /^-([AIMOBbqpiusv])(.*)/ or last;
 
         if ($1 eq 'I') {
             unshift @INC, $2;
@@ -317,6 +341,15 @@ if (!$start_pos or ($ARGV[0] eq '--par-options' && shift)) {
         }
 
         shift(@ARGV);
+
+        if (my $cmd = $dist_cmd{$1}) {
+            delete $ENV{'PAR_TEMP'};
+            init_inc();
+            require PAR::Dist;
+            &{"PAR::Dist::$cmd"}() unless @ARGV;
+            &{"PAR::Dist::$cmd"}($_) for @ARGV;
+            exit;
+        }
     }
 }
 
@@ -357,6 +390,7 @@ if ($out) {
     if ($bundle) {
         require PAR::Heavy;
         PAR::Heavy::_init_dynaloader();
+        init_inc();
         require_modules();
 
         my @inc = sort {
@@ -369,6 +403,8 @@ if ($out) {
              $_ ne $Config::Config{privlibexp});
         } @INC;
 
+        if ($^O eq 'MSWin32') { s{\\}{/}g for @inc }
+
         my %files;
         /^_<(.+)$/ and $files{$1}++ for keys %::;
         $files{$_}++ for values %INC;
@@ -384,7 +420,7 @@ if ($out) {
                     $file = $_;
                     last;
                 }
-                elsif (/^(\Q$dir\E\/(.*[^Cc]))\Z/) {
+                elsif (/^(\Q$dir\E\/(.*[^Cc]))\Z/i) {
                     ($file, $name) = ($1, $2);
                     last;
                 }
@@ -489,13 +525,13 @@ if ($out) {
     outs(qq(\$ENV{PAR_TEMP} = "$ENV{PAR_TEMP}"));
 
     foreach my $member ( $zip->members ) {
+        next if $member->isDirectory or !$ENV{PAR_TEMP};
         my $member_name = $member->fileName;
-        if ( $member_name =~ m|^/?shlib/(.+)$| and $ENV{PAR_TEMP} ) {
-            my $extract_name = $1;
-            my $dest_name = File::Spec->catfile($ENV{PAR_TEMP}, $extract_name);
-            $member->extractToFileNamed($dest_name);
-            outs(qq(Extracting "$member_name" to "$dest_name"));
-        }
+        next unless $member_name =~ m{^/?shlib/(?:$Config::Config{version}/)?(?:$Config::Config{archname}/)?([^/]+)$};
+        my $extract_name = $1;
+        my $dest_name = File::Spec->catfile($ENV{PAR_TEMP}, $extract_name);
+        $member->extractToFileNamed($dest_name);
+        outs(qq(Extracting "$member_name" to "$dest_name"));
     }
     # }}}
 }
@@ -533,6 +569,7 @@ sub require_modules {
     require Archive::Zip;
     require PAR;
     require PAR::Heavy;
+    require PAR::Dist;
 }
 
 # N.B. we set PAR_TMP_DIR and PAR_TEMP in myldr/main.c
@@ -613,10 +650,19 @@ sub pod_strip {
     }gsex;
     $pl_text = '#line 1 "' . ($filename) . "\"\n" . $pl_text
         if length $filename;
+    $pl_text =~ s/^#line 1 (.*\n)(#!.*\n)/$2#line 2 $1/g;
 
     return $pl_text;
 }
 
+sub init_inc {
+    require Config;
+    push @INC, grep defined, map $Config::Config{$_}, qw(
+        archlibexp privlibexp sitearchexp sitelibexp
+        vendorarchexp vendorlibexp
+    );
+}
+
 ########################################################################
 # The main package for script execution
 
@@ -635,7 +681,7 @@ exit;
 
 =head1 SEE ALSO
 
-L<PAR>, L<parl>, L<pp>
+L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 
 =head1 AUTHORS
 
